# Amethyst - An experimental Minecraft server written in Elixir.
# Copyright (C) 2024 KodiCraft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

defmodule Amethyst.Server.Stage1 do
  @moduledoc """
  This module contains the stage 1 (Handshaking) server logic.
  """
  alias Amethyst.Minecraft.Read

  def serve(client) do

  end

  ## DESERIALIZATION
  # Handshake https://wiki.vg/Protocol#Handshake
  def deserialize(0x00, <<data::binary>>) do
    {[ver, addr, port, next], ""} = Read.start(data) |> Read.varint() |> Read.string() |> Read.ushort() |> Read.varint() |> Read.stop()
    next = case next do
      1 -> :status
      2 -> :login
      3 -> :transfer
      _ -> raise RuntimeError, "Client requested moving to an unknown state!"
    end
    %{type: :handshake, version: ver, address: addr, port: port, next: next}
  end
  def deserialize(<<type, _::binary>>) do
    raise RuntimeError, "Got unknown packet type #{type}!"
  end
end
